@@ -19,7 +19,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <strings.h>
 #include <time.h>
 #include <unistd.h>
 #include <sys/time.h>
@@ -60,13 +59,11 @@ void ms_delay(int ms)
 /*
  *	Output a line and delay if needed.
  */
-void lineout(char const *line)
+static void lineout(char const *line, int len)
 {
   int ret;
 
   if (!cdelay) {
-    int len = strlen(line);
-
     do {
       ret = write(STDOUT_FILENO, line, len);
       if (ret < 0) {
@@ -129,9 +126,10 @@ void check_answer(void)
   tv.tv_usec = 0;
   while (select (STDIN_FILENO + 1, &rfds, NULL, NULL, &tv) > 0) {
     n = read (STDIN_FILENO, line, sizeof(line));
-    write(STDERR_FILENO, line, n);
-    FD_ZERO (&rfds);
-    FD_SET (STDIN_FILENO, &rfds);
+    if (write(STDERR_FILENO, line, n) == -1)
+      break;
+    FD_ZERO(&rfds);
+    FD_SET(STDIN_FILENO, &rfds);
     tv.tv_sec = 0;
     tv.tv_usec = 0;
   }
@@ -146,26 +144,36 @@ int asend(char *file)
   char line[1024];
   char *s;
   int first = 1;
+  long cur, len;
 
   if ((fp = fopen(file, "r")) == NULL) {
     perror(file);
     return -1;
   }
 
-  while (fgets(line, sizeof(line) - 1, fp) != NULL) {
+  cur = 0;
+
+  while (fgets(line, sizeof(line) - 2, fp) != NULL) {
+    long c = ftell(fp);
+    len = c - cur;
+    cur = c;
     if (dotrans && (s = strrchr(line, '\n')) != NULL) {
       /* s now points to \n */
       /* if there's a \r before, go there */
       if (s > line && *(s - 1) == '\r')
-        s--;
+        {
+          s--;
+	  len--;
+	}
       /* end of line */
       *s++ = '\r';
       *s++ = '\n';
       /* terminate string */
       *s = 0;
+      len++;
     }
-    lineout(line);
-    bdone += strlen(line);
+    lineout(line, len);
+    bdone += len;
     if (ldelay)
       ms_delay(ldelay);
     stats(first);
